/*
	CHANGELOG

	2007.06.21 - Stan Green
	- Performance enhancements for how the image is displayed.
	- File chooser dialogue to choose destination location.
	
	2007.07.01 - Dave Hope
	- File format now derived from the file extension chosen
	  when saving the file.
*/
import java.awt.*;
import java.awt.image.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import java.util.*;
import javax.imageio.*;
import javax.swing.*;
 
public class WaterMark extends JFrame
{
	/*
	 * Widgets.
	 */
	JTextField		SourceFileBox			=	new JTextField( 15 );
	JTextField		WatermarkFileBox		=	new JTextField( 15 );
	JFileChooser	SourceFileChooser		=	new JFileChooser();
	JFileChooser	WatermarkFileChooser	=	new JFileChooser();	
	JButton			ButtonCreate			=	new JButton( "Create" );
	JButton			SourceButton			=	new JButton( "Select Source Image" );
	JButton			WaterMarkButton			=	new JButton( "Select Watermark Image" );
	
	String			ChosenFormat			=	"png";
	
	String[]		WatermarkPositions		=	{ "centre", "bottom right" };
	int				ChosenPosition			=	1;
	JComboBox		PositionChooser			=	new JComboBox( WatermarkPositions );
	
	JLabel			LabelFormat				=	new JLabel( "Output Format:" );
	JLabel			LabelPosition			=	new JLabel( "Watermark Position:" );
	
	
	/*
	 * Constructor.
	 * Build the awful swing UI.
	 */
	WaterMark()
	{
		// Disable Filename Boxes.
		SourceFileBox.setEditable( false );
		WatermarkFileBox.setEditable( false );
		
		/*
		 * Assign Actions.
		 */
		SourceButton.addActionListener( new SourceButtonAction() );
		WaterMarkButton.addActionListener( new WaterMarkButtonAction() );
		ButtonCreate.addActionListener( new ButtonCreateAction() );
		PositionChooser.addActionListener( new PositionChooserAction() );
		
		/*
 		 * Create Content Pane.
 		 */
 		JPanel ContentArea			=	new JPanel( ); 
		ContentArea.setLayout( new GridLayout(11, 0) );
		ContentArea.add( SourceFileBox );
		ContentArea.add( SourceButton );
		ContentArea.add( Box.createVerticalGlue() );

		ContentArea.add( WatermarkFileBox );
		ContentArea.add( WaterMarkButton );
		ContentArea.add( Box.createVerticalGlue() );
		
		ContentArea.add( LabelPosition );
		ContentArea.add( PositionChooser );
		ContentArea.add( Box.createVerticalGlue() );

		ContentArea.add( ButtonCreate );

		ContentArea.setBorder( BorderFactory.createEmptyBorder( 5, 5, 5, 5 ) );
		

    
		this.setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		this.setContentPane( ContentArea );
		this.pack();
		this.setTitle( "Watermark" );
		this.setResizable( false );
		this.setVisible( true );
	}
	
	
	/*
	 * Main function.
	 * Only here to call MakeWatermark() for portability reasons.
	 */
	public static void main( String[] args ) throws IOException
	{
	        JFrame MainWindow			=	new WaterMark();
	}
	
	
	/*
	 * SourceButtonAction.
	 * Gets the Source file path and updates the UI.
	 */
	class SourceButtonAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			int retval			=	SourceFileChooser.showOpenDialog( WaterMark.this );
			if (retval == JFileChooser.APPROVE_OPTION)
			{
				File file		=	SourceFileChooser.getSelectedFile();

				SourceFileBox.setText( file.getPath() );
			}
		}
	}
	
	
	/*
	 * WaterMarkButtonAction.
	 * Gets the Watermark file path and updates the UI.
	 */
	class WaterMarkButtonAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			int retval			=	WatermarkFileChooser.showOpenDialog( WaterMark.this );
			if (retval == JFileChooser.APPROVE_OPTION)
			{
				File file		=	WatermarkFileChooser.getSelectedFile();

				WatermarkFileBox.setText( file.getPath() );
			}
		}
	}	
	
	/*
	 * PositionChooserAction.
	 * Allows the user to select the position of the watermark.
	 */
	class PositionChooserAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{
			JComboBox tmpSelection		=	(JComboBox)ae.getSource();
			String _ChosenPosition		=	(String)tmpSelection.getSelectedItem();
			
			if ( _ChosenPosition == "centre" )
			{
				ChosenPosition		=	1;
			}
			else
			{
				ChosenPosition		=	2;
			}
		}
	}
	
	/*
	 * ButtonCreateAction.
	 *
	 */
	class ButtonCreateAction implements ActionListener
	{
		public void actionPerformed(ActionEvent ae)
		{	
			try
			{

				// Prompt user to choose source file.
				if( SourceFileBox.getText().length() == 0 )
				{
					JOptionPane.showMessageDialog
					(
						WaterMark.this,
						"Please select a source image.",
						"WaterMark",
						JOptionPane.ERROR_MESSAGE
					);
					return;
				}
				
				// Prompt user to choose watermark file.
				if( WatermarkFileBox.getText().length() == 0 )
				{
					JOptionPane.showMessageDialog
					(
						WaterMark.this,
						"Please select a watermark image.",
						"WaterMark",
						JOptionPane.ERROR_MESSAGE
					);
					return;
				}
				
				// Build output file.
				JFileChooser OutputFileChooser	=	new JFileChooser();
				OutputFileChooser.setCurrentDirectory(new File("."));
				String	OutputFile				=	SourceFileBox.getText();
				int lastDot						=	OutputFile.lastIndexOf('.');
				if(lastDot > 0)
				{
					ChosenFormat	=	OutputFile.substring(lastDot + 1, OutputFile.length());
					OutputFile		=	OutputFile.substring(0, lastDot + 1) + "WM." + ChosenFormat;
				}
				else
				{
					OutputFile		=	OutputFile + ".WM." + ChosenFormat;
				}

				OutputFileChooser.setSelectedFile(new File(OutputFile));
				boolean fileExists = true;
				
				int retval = 0;
				do
				{
					retval			=	OutputFileChooser.showSaveDialog( WaterMark.this );
												
					if (retval == JFileChooser.APPROVE_OPTION)
					{
						File file =	OutputFileChooser.getSelectedFile();
						OutputFile = file.getPath();
						
						fileExists = new File(OutputFile).exists();
						
						if(fileExists)
						{
							retval = JOptionPane.showConfirmDialog
							(
								WaterMark.this,
								"The file " + OutputFile + " already exists.\n" +
								" Are you sure you want to overwrite it?",
								"WaterMark",
								JOptionPane.YES_NO_OPTION
							);
							
							if(retval == JOptionPane.YES_OPTION)
							{
								fileExists = false;
							}
						}
					}
					else
					{
						fileExists = false;
						return;
					}
				}
				while (fileExists);
				
				/*
				 * Call MakeWatermark.
				 */
				BufferedImage OutImage = MakeWatermark(
					SourceFileBox.getText(),
					WatermarkFileBox.getText(),
					OutputFile,
					ChosenPosition
				);
				
				/*
				 * Display Image.
				 */
				JFrame ShowOutput	=	new JFrame("WaterMark: " + OutputFile );
				ShowOutput.getContentPane().add( new JLabel( new ImageIcon( OutImage ) ) );
				ShowOutput.pack();
				ShowOutput.setLocationRelativeTo( null );
				ShowOutput.setVisible( true );		
				
			}
			catch( IOException e)
			{
				// Do nothing
			}
		}
	}
	
	
	/*
	 * MakeWatermark( Source, Watermark, Output, Position).
	 * Creates a watermarked image.
	 */
	 public static BufferedImage MakeWatermark( String InputA, String InputB, String OutFile, int WatermarkPosition ) throws IOException
	{
		BufferedImage SourceImage = null;
		String OutFormat = null;
		
		try
		{
			/*
			 * Determine output format.
			 * Should probably check to make sure they're entering something sane...
			 */
			int lastDot						=	OutFile.lastIndexOf('.');
			if(lastDot > 0)
			{
				OutFormat	=	OutFile.substring(lastDot + 1, OutFile.length());
			}
			
			/*
			 * Load Source Image.
			 */
			File SourceFile					=	new File( InputA );
			SourceImage 					=	ImageIO.read(SourceFile);
			
			/*
			 * Load Watermark Image.
			 */
			File WatermarkFile				=	new File( InputB );
			BufferedImage WatermarkImage	=	ImageIO.read( WatermarkFile );
			
			
			/*
			 * Create canvas.
			 */
			Graphics2D Canvas				=	SourceImage.createGraphics();
			Canvas.setComposite( AlphaComposite.getInstance( AlphaComposite.SRC_OVER, 0.4f ) );
			
			
			/*
			 * Determine Watermark position.
			 *
			 * 1: Center.
			 * 2: Bottom Right.
			 */
			int _WatermarkPosition;
			int WaterMarkX;
			int WaterMarkY;

			switch ( WatermarkPosition )
			{					
				case 1:
				WaterMarkX		=	( SourceImage.getWidth() - WatermarkImage.getWidth() ) / 2;
				WaterMarkY		=	( SourceImage.getHeight() - WatermarkImage.getHeight() ) / 2;
				break;
							
				case 2:
				WaterMarkX		=	( SourceImage.getWidth() - WatermarkImage.getWidth() );
				WaterMarkY		=	( SourceImage.getHeight() - WatermarkImage.getHeight() );
				break;
				
				default:
				WaterMarkX		=	( SourceImage.getWidth() - WatermarkImage.getWidth() ) / 2;
				WaterMarkY		=	( SourceImage.getHeight() - WatermarkImage.getHeight() ) / 2;
				break;
			}
			
					
			/*
			 * Draw watermark on canvas.
			 */
			Canvas.drawImage( WatermarkImage, WaterMarkX, WaterMarkY, null );
			
			
			/*
			 * Clean up.
			 */
			Canvas.dispose();
 		 	
			/*
			 * Write out the file.
			 */
			ImageIO.write(
				SourceImage,
				OutFormat,
				new File( OutFile )
			);
			
		}
		catch ( IOException e )
		{
			System.out.println("Error: " + e );
			// Do nothing
		}

		return(SourceImage);
	}
}
